import { Observable } from 'rxjs'
import { Auth } from 'aws-amplify'
import { from } from 'rxjs'

interface IAuthService {
    
    signUp: (
        username: string,
        password: string,
        email: string,
        givenName: string,
        familyName: string,
    ) => Observable<any>;
   
    confirmSignUp: (
        username: string,
        code: string,
    ) => Observable<any>;
   
    signUpSuccessfulSignIn: (
        username: string,
        password: string,
        name: string,
    ) => Observable<any>;
    
    resendConfirmationCode: (
        username: string,
    ) => Observable<any>;
   
    signIn: (username: string, password: string) => Observable<any>;
    
    newPasswordRequired: (
        user: any,
        newPassword: string,
    ) => Observable<any>;

    signOut: () => Observable<any>;

    forgotPassword: (username: string) => Observable<any>;

    forgotPasswordSubmit: (
        username: string,
        code: string,
        password: string,
    ) => Observable<any>;

    changeAttributes: (
        email: string,
        name: string,
        id: number,
    ) => Observable<any>;

    changePassword: (
        user: any,
        oldPassword: string,
        newPassword: string,
    ) => Observable<string>;

}

class AuthService implements IAuthService {
    signUp = (
        username: string,
        password: string,
        email: string,
        givenName: string,
        familyName: string,
    ) => from(
        Auth.signUp({
            'username': username,
            'password': password,
            'attributes': {
                'email': email,
                'given_name': givenName,
                'family_name': familyName,
            }
        })
    )

    confirmSignUp = (
        username: string,
        code: string,
    ) => from(
        Auth.confirmSignUp(username, code)
    )

    signUpSuccessfulSignIn = (
        username: string,
        password: string,
    ) => from(
        Auth.signIn(username, password)
    )

    resendConfirmationCode = (
        username: string,
    ) => from(
        Auth.resendSignUp(username)
    )

    newPasswordRequired = (
        user: any,
        newPassword: string,
    ) => from(
        Auth.completeNewPassword(
            user,
            newPassword,
            {}
        )
    ).mergeMap(r => from(Auth.currentUserInfo()))

    signIn = (
        username: string, 
        password: string
    ) => from(
        Auth.signIn(
            username,
            password,
        )
    )

    signOut = () => from(
        Auth.signOut()
    )
 
    forgotPassword = (
        username: string,
    ) => from(Auth.forgotPassword(
        username,
    ))

    forgotPasswordSubmit = (
        username: string,
        code: string,
        password: string,
    ) => from(Auth.forgotPasswordSubmit(
        username,
        code,
        password,
    ))

    changeAttributes = (
        email: string,
        name: string,
        guestId: number,
    ) => from(Auth.currentAuthenticatedUser()
    ).mergeMap(user => 
        from(Auth.updateUserAttributes(
            user,
            { email } 
    )))

    changePassword = (
        oldPassword: string,
        newPassword: string,
    ) => from(Auth.currentAuthenticatedUser()
    ).mergeMap(user => from(Auth.changePassword(
        user,
        oldPassword,
        newPassword,
    )))
    
}

export default new AuthService()