import * as React from 'react'
import { useDispatch, useSelector } from 'react-redux'
import authDispatches from './dispatches'

const AuthContext = React.createContext<any>(undefined)

interface IAuthProvider {}
function AuthProvider(props: IAuthProvider) {
    const dispatch = useDispatch()
    const isAuthenticated = useSelector((state: any) => state.authReducer.isAuthenticated)
    const status = useSelector((state: any) => state.authReducer.status)

    const dispatches = authDispatches(dispatch)
    
    return <AuthContext.Provider value={{dispatches, isAuthenticated, status}} {...props} />
}

function useAuthDispatch() {
    const context = React.useContext(AuthContext)
    if (!context) {
        throw new Error('useAuthDispatch must be used within AuthProvider')
    }
    return context.dispatches
}

function isAuthenticated() {
    const context = React.useContext(AuthContext)
    if (!context) {
        throw new Error('isAuthenticated must be used within AuthProvider')
    }
    return context.isAuthenticated
}

function useStatusData() {
    const context = React.useContext(AuthContext)
    if (!context) {
        throw new Error('useStatusData must be used within AuthProvider')
    }
    return context.status
}

export { AuthProvider, useAuthDispatch, isAuthenticated , useStatusData}