import { AUTH_ACTION_TYPES } from "./types";

interface IActionBase<T> {
    type: T
}

export interface IResponse<T, R> {
    type: T,
    response: R,
}

export interface IError<T, E>{
    type: T,
    error: E,
}
  

export interface ISignUpAction {
    email: string;
    password: string;
    firstName: string;
    lastName: string
}  
  
export interface ISignUp extends IActionBase<AUTH_ACTION_TYPES.SIGN_UP>, ISignUpAction {}

export interface IConfirmSignUp extends IActionBase<AUTH_ACTION_TYPES.CONFIRM_SIGN_UP> {
    confirmationCode: string;
}
  
export interface IResendConfirmationCode extends IActionBase<AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE> {
    email: string;
}  
export interface IResendConfirmationCodeSubmitted extends IActionBase<AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_SUBMITTED> {
    response: any;
}

export interface IResendConfirmationCodeError extends IActionBase<AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_ERROR> {
    error: any;
}

export interface IResendConfirmationCodeFailed extends IActionBase<AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_FAILED> {
    error: any;
}

export interface ISignUpSubmitted extends IActionBase<AUTH_ACTION_TYPES.SIGN_UP_SUBMITTED> {
    response: any; 
}

export interface ISignUpSuccessful extends IActionBase<AUTH_ACTION_TYPES.SIGN_UP_SUCCESSFUL> {
    response: any; 
}

export interface ISignUpFailed extends IActionBase<AUTH_ACTION_TYPES.SIGN_UP_FAILED> {
    error: any; 
}

export interface ISignInAction {
    email: string;
    password: string;
}

export interface ISignInSuccessful extends IActionBase<AUTH_ACTION_TYPES.SIGN_IN_SUCCESSFUL> {
    response: any; 
}

export interface ISignIn extends IActionBase<AUTH_ACTION_TYPES.SIGN_IN>, ISignInAction {}


export interface ISignInFailed extends IActionBase<AUTH_ACTION_TYPES.SIGN_IN_FAILED> {
    error: any; 
}

export interface ISignOut extends IActionBase<AUTH_ACTION_TYPES.SIGN_OUT> {}


export interface INewPasswordRequired extends IActionBase<AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED> {
    response: any,
}

export interface INewPasswordRequiredFailed extends IActionBase<AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED_FAILED> {
    error: any,
}

export interface IForgotPassword extends IActionBase<AUTH_ACTION_TYPES.FORGOT_PASSWORD> {
    email: string;
}

export interface IForgotPasswordSent extends IActionBase<AUTH_ACTION_TYPES.FORGOT_PASSWORD_SENT> {
    response: any;
}

export interface IForgotPasswordSuccessful extends IActionBase<AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUCCESSFUL> {
    response: any;
}

export interface IForgotPasswordFailed extends IActionBase<AUTH_ACTION_TYPES.FORGOT_PASSWORD_FAILED> {
    error: any;
}

export interface IForgotPasswordSubmit extends IActionBase<AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUBMIT> {
    code: string;
    newPassword: string;
}

export interface IChangePassword extends IActionBase<AUTH_ACTION_TYPES.CHANGE_PASSWORD> {
    oldPassword: string;
    newPassword: string;
}

export interface IChangePasswordSuccessful extends IActionBase<AUTH_ACTION_TYPES.CHANGE_PASSWORD_SUCCESSFUL> {
    response: any;
}

export interface IChangePasswordFailed extends IActionBase<AUTH_ACTION_TYPES.CHANGE_PASSWORD_FAILED> {
    error: any;
}

export interface IChangeAttributes extends IActionBase<AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES> {
    name: string;
    email: string;
}

export interface IChangeAttributesSuccessful extends IActionBase<AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_SUCCESSFUL> {
    response: any;
}

export interface IChangeAttributesFailed extends IActionBase<AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_FAILED> {
    error: any;
}

export interface IUserNotConfirmed extends IActionBase<AUTH_ACTION_TYPES.USER_NOT_CONFIRMED> {
    error: any,
}