"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../actions/types");
const AuthState = {
    email: '',
    password: '',
    firstName: '',
    lastName: '',
    isAuthenticated: false,
    status: {
        type: types_1.AUTH_ACTION_TYPES.UNAUTHENTICATED,
        response: null,
        error: null,
    }
};
exports.default = (state = AuthState, action) => {
    switch (action.type) {
        case types_1.AUTH_ACTION_TYPES.SIGN_UP: {
            const { email, password, firstName, lastName, type } = action;
            return Object.assign(Object.assign({}, state), { email,
                password,
                firstName,
                lastName, status: {
                    type
                } });
        }
        case types_1.AUTH_ACTION_TYPES.SIGN_IN: {
            const { email, type } = action;
            return Object.assign(Object.assign({}, state), { email, status: {
                    type
                } });
        }
        case types_1.AUTH_ACTION_TYPES.SIGN_IN_SUCCESSFUL: {
            const { type, response } = action;
            return Object.assign(Object.assign({}, AuthState), { isAuthenticated: true, status: {
                    type,
                    response,
                } });
        }
        case types_1.AUTH_ACTION_TYPES.SIGN_OUT: {
            return Object.assign(Object.assign({}, AuthState), { isAuthenticated: false, status: {
                    type: types_1.AUTH_ACTION_TYPES.UNAUTHENTICATED
                } });
        }
        case types_1.AUTH_ACTION_TYPES.FORGOT_PASSWORD: {
            const { email, type } = action;
            return Object.assign(Object.assign({}, state), { email, status: {
                    type
                } });
        }
        case types_1.AUTH_ACTION_TYPES.SIGN_UP_SUBMITTED:
        case types_1.AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_SUBMITTED:
        case types_1.AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED:
        case types_1.AUTH_ACTION_TYPES.FORGOT_PASSWORD_SENT:
        case types_1.AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUCCESSFUL:
        case types_1.AUTH_ACTION_TYPES.CHANGE_PASSWORD_SUCCESSFUL:
        case types_1.AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_SUCCESSFUL:
            {
                const { type, response } = action;
                return Object.assign(Object.assign({}, state), { status: {
                        type,
                        response,
                    } });
            }
        case types_1.AUTH_ACTION_TYPES.SIGN_UP_FAILED:
        case types_1.AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_ERROR:
        case types_1.AUTH_ACTION_TYPES.SIGN_IN_FAILED:
        case types_1.AUTH_ACTION_TYPES.USER_NOT_CONFIRMED:
        case types_1.AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED_FAILED:
        case types_1.AUTH_ACTION_TYPES.FORGOT_PASSWORD_FAILED:
        case types_1.AUTH_ACTION_TYPES.CHANGE_PASSWORD_FAILED:
        case types_1.AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_FAILED:
            {
                const { type, error } = action;
                return Object.assign(Object.assign({}, state), { status: {
                        type,
                        error,
                    } });
            }
        default: {
            return state;
        }
    }
};
