import { ofType, Epic } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../actions/types'
import AuthService from '../../AuthService'

type ChangeAttributesAction = {
    type: string,
    name: string,
    email: string,
}

const changeAttributes: Epic<ChangeAttributesAction, any, any> = (action$, state$) =>
    action$.pipe(
        ofType(AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES),
        switchMap(action => {
            const name = action.name
            const email = action.email
            const userId = state$.value.authReducer.userId
            return AuthService.changeAttributes(
                email,
                name,
                userId,
            ).mergeMap(response => Observable.of(
                { type: AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_SUCCESSFUL, response }
            )).catch(error => Observable.of(
                { type: AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_FAILED, error }
            ))
        })
    )

export default changeAttributes