import { ofType, Epic } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../../actions/types'
import AuthService from '../../../AuthService'

type ChangePasswordAction = {
    type: string,
    oldPassword: string,
    newPassword: string,
}

const changePassword: Epic<ChangePasswordAction, any, any> = (action$, state$) =>
    action$.pipe(
        ofType(AUTH_ACTION_TYPES.CHANGE_PASSWORD),
        switchMap(action => {
            const oldPassword = action.oldPassword
            const newPassword = action.newPassword
            return AuthService.changePassword(
                oldPassword,
                newPassword
            ).mergeMap(response => Observable.of(
                { type: AUTH_ACTION_TYPES.CHANGE_PASSWORD_SUCCESSFUL, response },
            ))
            .catch(error => Observable.of(
                { type:  AUTH_ACTION_TYPES.CHANGE_PASSWORD_FAILED, error },
            ))
        })
    )

export default changePassword