import { Epic, ofType } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../../actions/types'
import AuthService from '../../../AuthService'

type ForgotPasswordAction = {
    type: string,
    email: string,
}

const forgotPassword: Epic<ForgotPasswordAction, any, any> = (action$, state$) => {
    return action$.pipe(
        ofType(AUTH_ACTION_TYPES.FORGOT_PASSWORD),
        switchMap((action) =>
            AuthService.forgotPassword(action.email)
                .mergeMap(response => Observable.of(
                    { type: AUTH_ACTION_TYPES.FORGOT_PASSWORD_SENT, response}            
                )).catch(error => Observable.of(
                    { type: AUTH_ACTION_TYPES.FORGOT_PASSWORD_FAILED, error}            
                ))
            ))
}

export default forgotPassword