import { ofType, Epic } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../../actions/types'
import AuthService from '../../../AuthService'

type ForgotPasswordSubmitAction = {
    type: string,
    code: string,
    newPassword: string,
}

const forgotPasswordSubmit: Epic<ForgotPasswordSubmitAction, any, any> = (action$, state$) => {
    return action$.pipe(
        ofType(AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUBMIT),
        switchMap((action) =>
                AuthService.forgotPasswordSubmit(
                    state$.value.authReducer.email,
                    action.code,
                    action.newPassword
                ).mergeMap(response => Observable.of(
                    {
                        type: AUTH_ACTION_TYPES.SIGN_IN,
                        username: state$.value.authReducer.email,
                        password: action.newPassword
                    },
                    { type: AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUCCESSFUL, response}
                )).catch(error => Observable.of(
                    { type: AUTH_ACTION_TYPES.FORGOT_PASSWORD_FAILED, error}
                ))
            ))
}

export default forgotPasswordSubmit