import { Epic, ofType } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../../actions/types'
import AuthService from '../../../AuthService'

type NewPasswordRequiredAction = {
    type: string,
    password: string,
}

const newPasswordRequired: Epic<NewPasswordRequiredAction, any, any> = (action$, state$) =>
    action$.pipe(
        ofType(AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED),
        switchMap((action) =>
            AuthService.newPasswordRequired(
                state$.value.authReducer.user,
                action.password,
            ).mergeMap(response => Observable.of(
                { type: AUTH_ACTION_TYPES.SIGN_IN_SUCCESSFUL, response }
            )).catch(error => Observable.of(
                { type: AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED_FAILED, error}
            ))
        )
    )

export default newPasswordRequired