import { Epic, ofType } from 'redux-observable'
import { switchMap } from 'rxjs/operators'
import { Observable } from 'rxjs/Observable'
import 'rxjs/add/observable/from'
import 'rxjs/add/observable/of'
import 'rxjs/add/operator/mergeMap'
import 'rxjs/add/operator/catch'
import { AUTH_ACTION_TYPES } from '../../../actions/types'
import AuthService from '../../../AuthService'

type ConfirmSignAction = {
    type: string,
    confirmationCode: string,
}

const confirmSignup: Epic<ConfirmSignAction, any, any> = (action$, state$) =>
    action$.pipe(
        ofType(AUTH_ACTION_TYPES.CONFIRM_SIGN_UP),
        switchMap((action) =>
            AuthService.confirmSignUp(
                state$.value.authReducer.email,
                action.confirmationCode,
            ).mergeMap(response => {
                return Observable.of(
                    { type: AUTH_ACTION_TYPES.SIGN_UP_SUCCESSFUL, response },
                )}).catch(error => {
                    if (error.code === 'ExpiredCodeException') {
                        return  Observable.of(
                            {type: AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_ERROR, error}
                        )
                    } else {
                        return Observable.of(
                            {type: AUTH_ACTION_TYPES.SIGN_UP_FAILED, error}
                        )
                    }
            }))
    )

export default confirmSignup