import { AUTH_ACTION_TYPES } from "../actions/types"
import { IChangeAttributes, IChangePassword, IConfirmSignUp, 
  IForgotPassword, IForgotPasswordSubmit, IResendConfirmationCode, ISignIn, ISignOut, ISignUp, 
  ISignUpSuccessful, ISignInSuccessful, ISignUpSubmitted, ISignUpFailed, IResendConfirmationCodeError, IResendConfirmationCodeSubmitted,
  ISignInFailed, INewPasswordRequired, IUserNotConfirmed, INewPasswordRequiredFailed, IForgotPasswordSent, IForgotPasswordFailed,
  IForgotPasswordSuccessful, IChangePasswordSuccessful, IChangePasswordFailed, IChangeAttributesSuccessful, IChangeAttributesFailed, 
} from '../actions/interfaces'

type STATUS_TYPES = AUTH_ACTION_TYPES

interface IAuthState {
  email: string;
  password: string;
  firstName: string;
  lastName: string;  

  isAuthenticated: boolean;
  status: {
    type: STATUS_TYPES;
    response: any;
    error: any;
  }

}

const AuthState: IAuthState = {
  email: '',
  password: '',
  firstName: '',
  lastName: '',

  isAuthenticated: false,
  status: {
    type: AUTH_ACTION_TYPES.UNAUTHENTICATED,
    response: null,
    error: null,
  }
}

type IAction =  IChangeAttributes | IChangePassword | IConfirmSignUp | IForgotPassword | 
                IForgotPasswordSubmit | IResendConfirmationCode | ISignIn | ISignOut | ISignUp |
                ISignUpSuccessful | ISignInSuccessful | ISignUpSubmitted | ISignUpFailed | IResendConfirmationCodeError |
                IResendConfirmationCodeSubmitted | ISignInFailed | INewPasswordRequired | IUserNotConfirmed | INewPasswordRequiredFailed |
                IForgotPasswordSent | IForgotPasswordFailed | IForgotPasswordSuccessful | IChangePasswordSuccessful | IChangePasswordFailed |
                IChangeAttributesSuccessful | IChangeAttributesFailed 

export default (state: IAuthState = AuthState, action: IAction) => {
  switch (action.type) {

    case AUTH_ACTION_TYPES.SIGN_UP: {
      const { email, password, firstName, lastName, type } = action
      return {
        ...state,
        email,
        password,
        firstName,
        lastName,

        status: {
          type
        }
      }
    }

    case AUTH_ACTION_TYPES.SIGN_IN: {
      const { email, type } = action
      return {
        ...state,
        email,
        status: {
          type
        }

      }
    }

    case AUTH_ACTION_TYPES.SIGN_IN_SUCCESSFUL: {
      const { type, response } = action
      return {
        ...AuthState,

        isAuthenticated: true,
        status: {
          type,
          response,
        }
      }
    }

    case AUTH_ACTION_TYPES.SIGN_OUT: {
      return {
        ...AuthState,

        isAuthenticated: false,
        status: {
          type: AUTH_ACTION_TYPES.UNAUTHENTICATED
        }
      }
    }

    case AUTH_ACTION_TYPES.FORGOT_PASSWORD: {
      const { email, type } = action
      return {
        ...state,
        email,
        status: {
          type
        },
      }
    }

    case AUTH_ACTION_TYPES.SIGN_UP_SUBMITTED:
    case AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_SUBMITTED:
    case AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED: 
    case AUTH_ACTION_TYPES.FORGOT_PASSWORD_SENT:
    case AUTH_ACTION_TYPES.FORGOT_PASSWORD_SUCCESSFUL:
    case AUTH_ACTION_TYPES.CHANGE_PASSWORD_SUCCESSFUL:
    case AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_SUCCESSFUL:
    {
      const { type, response } = action
      return {
        ...state,
        status: {
          type,
          response,
        }
      }
    }


    case AUTH_ACTION_TYPES.SIGN_UP_FAILED:
    case AUTH_ACTION_TYPES.RESEND_CONFIRMATION_CODE_ERROR:
    case AUTH_ACTION_TYPES.SIGN_IN_FAILED:
    case AUTH_ACTION_TYPES.USER_NOT_CONFIRMED: 
    case AUTH_ACTION_TYPES.NEW_PASSWORD_REQUIRED_FAILED: 
    case AUTH_ACTION_TYPES.FORGOT_PASSWORD_FAILED:
    case AUTH_ACTION_TYPES.CHANGE_PASSWORD_FAILED:
    case AUTH_ACTION_TYPES.CHANGE_ATTRIBUTES_FAILED:
    {
      const { type, error } = action
      return {
        ...state,
        status: {
          type,
          error,
        }
      }
    }
  
    default: {
      return state
    }
  }
}